@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 from itertools import combinations
 from io import BytesIO, StringIO
-from previsionio.usecase_config import UsecaseState
+from previsionio.experiment_config import ExperimentState
 import numpy as np
 import pandas as pd
 import os
@@ -25,7 +25,7 @@ class Dataset(ApiResource):
 
     """ Dataset objects represent data resources that will be explored by Prevision.io platform.
 
-        In order to launch an auto ml process (see :class:`.BaseUsecase` class), we need to have
+        In order to launch an auto ml process (see :class:`.BaseExperiment` class), we need to have
         the matching dataset stored in the related workspace.
 
         Within the platform they are stored in tabular form and are derived:
@@ -38,7 +38,7 @@ class Dataset(ApiResource):
     def __init__(self, _id: str, name: str, datasource: DataSource = None, _data: DataFrame = None,
                  describe_state: Dict = None, drift_state=None, embeddings_state=None, separator=',', **kwargs):
 
-        super().__init__(_id=_id, datasource=datasource)
+        super().__init__(_id=_id)
         self.name = name
         self._id = _id
         self.separator = separator
@@ -105,13 +105,13 @@ class Dataset(ApiResource):
         self.embeddings_state = dset_json['embeddings_state']
 
     def get_describe_status(self):
-        return UsecaseState(self.describe_state)
+        return ExperimentState(self.describe_state)
 
     def get_drift_status(self):
-        return UsecaseState(self.drift_state)
+        return ExperimentState(self.drift_state)
 
     def get_embedding_status(self):
-        return UsecaseState(self.embeddings_state)
+        return ExperimentState(self.embeddings_state)
 
     def delete(self):
         """Delete a dataset from the actual [client] workspace.
@@ -311,14 +311,12 @@ class Dataset(ApiResource):
 
         create_json = parse_json(create_resp)
         url = '/{}/{}'.format(cls.resource, create_json['_id'])
-        event_tuple = previsionio.utils.EventTuple('DATASET_UPDATE',
-                                                   [('copy_state', 'done'),
-                                                       ('describe_state', 'done'),
-                                                       ('drift_state', 'done')],
-                                                   [('copy_state', 'failed'),
-                                                       ('describe_state', 'failed'),
-                                                       ('drift_state', 'failed'),
-                                                       ('embeddings_state', 'failed')])
+        event_tuple = previsionio.utils.EventTuple(
+            'DATASET_UPDATE',
+            [('copy_state', 'done'), ('describe_state', 'done'), ('drift_state', 'done')],
+            [('copy_state', 'failed'), ('describe_state', 'failed'),
+             ('drift_state', 'failed'), ('embeddings_state', 'failed')],
+        )
         assert pio.client.event_manager is not None
         pio.client.event_manager.wait_for_event(create_json['_id'],
                                                 cls.resource,
@@ -339,7 +337,7 @@ class DatasetImages(ApiResource):
     """ DatasetImages objects represent image data resources that will be used by
         Prevision.io's platform.
 
-        In order to launch an auto ml process (see :class:`.BaseUsecase` class), we need to have
+        In order to launch an auto ml process (see :class:`.BaseExperiment` class), we need to have
         the matching dataset stored in the related workspace.
 
         Within the platform, image folder datasets are stored as ZIP files and are copied from
